##############################################################################
# CustomClass Rules                                                          #
# =================                                                          #
#                                                                            #
#1. All custom classes must inherit sreb.EBObject and the constructor        #
#   must call sreb.EBObject's constructor. If a class starts with _ then it  #
#   is considered internal and will not be treated as a custom class.        #
#                                                                            #
#2. The custom class will only use the default constructor.                  #  
#   ie. a constructor with no parameters.                                    #
#                                                                            #
#3. To add a property provide getter and have an attribute that does not     #
#   starts with an underscore(_) or an upper case letter, and have the       #
#   attribute of a know type. Known types are int,float,str,EBPoint,EBColor, #
#   tuple, and list.                                                         #
#   If an attribute's default value is of type tuple and only has two        #
#   items, the property will be treated as an EBPoint. Similarly, if an      #
#   attribute's default value is a tuple of 3 items the property will be     #
#   treated as an EBColor.  The input type of the setter and the output type #
#   of the getter is expected to be the same as the type of the attribute.   #
#                                                                            #
#4. If only getter is provided, the property will be a readonly property.    # 
#                                                                            #
#6. The custom class may be instanciated during development to obtain        # 
#   class info. Avoid such things as display mode change, remote connections # 
#   in the constructor.                                                      #
#                                                                            # 
#7. Any method in the custom class can be called using the Execute action    #
#   By default, the return type of the method is string unless a doc string  #
#   with the following constraint is available                               #
#	a. The doc string starts with "RETURN:" (case matters)               #
#       b. Following the text "RETURN:" provide a default value of the type  #
#          or the __repr__ value of the class. eg. str for string            #
#8. If a property's setter metthod has default values for it's parameters,   #
#    the property will not accept references or equation.                    #
##############################################################################


import sreb

class CustomClassTemplate(sreb.EBObject):
	def __init__(self):
		sreb.EBObject.__init__(self)
		# Creating a dictionary, containing the square coordinates as key and a list containing ability to move and the index of the piece as value
		self.squareDict = dict() 
		self.screenres = 0.9185185185185185185185185185185185
		for i in range(8):
			self.squareDict[(self.screenres*67.5, (1080-(self.screenres*67.5+i*self.screenres*135+44)))] = [('A'+str(i))]
			self.squareDict[(self.screenres*202.5, (1080-(self.screenres*67.5+i*self.screenres*135+44)))] = [('B'+str(i))]
			self.squareDict[(self.screenres*337.5, (1080-(self.screenres*67.5+i*self.screenres*135+44)))] = [('C'+str(i))]
			self.squareDict[(self.screenres*472.5, (1080-(self.screenres*67.5+i*self.screenres*135+44)))] = [('D'+str(i))]
			self.squareDict[(self.screenres*607.5, (1080-(self.screenres*67.5+i*self.screenres*135+44)))] = [('E'+str(i))]
			self.squareDict[(self.screenres*742.5, (1080-(self.screenres*67.5+i*self.screenres*135+44)))] = [('F'+str(i))]
			self.squareDict[(self.screenres*877.5, (1080-(self.screenres*67.5+i*self.screenres*135+44)))] = [('G'+str(i))]
			self.squareDict[(self.screenres*1012.5,(1080-(self.screenres*67.5+i*self.screenres*135+44)))] = [('H'+str(i))]
			self.squareDict[(self.screenres*1147.5,(1080-(self.screenres*67.5+i*self.screenres*135+44)))] = ['Start_1']
			self.squareDict[(self.screenres*1282.5,(1080-(self.screenres*67.5+i*self.screenres*135+44)))] = ['Start_2']
			self.squareDict[(self.screenres*1417.5,(1080-(self.screenres*67.5+i*self.screenres*135+44)))] = ['Start_3']
			self.squareDict[(self.screenres*1552.5,(1080-(self.screenres*67.5+i*self.screenres*135+44)))] = ['Start_4']

		# List containing the locations of all pieces
		self.positionList = list()
		self.source = None   # Initializing the position the piece comes from	
		self.destine = None  # Initializing the position the piece comes from
		self.visibleList = [True, True, True, True,True, True, True, True,True, True, True, True,True, True, True, True,
		True, True, True, True,True, True, True, True,True, True, True, True,True, True, True, True]
		self.moveableList = [True, True, True, True,True, True, True, True,True, True, True, True,True, True, True, True,
		True, True, True, True,True, True, True, True,True, True, True, True,True, True, True, True] # 
		self.completed = 0
		self.moveCount = 0
		# List of the square position on x-axis
		
		self.xSquarePositionList = [self.screenres*67.5, self.screenres*202.5, self.screenres*337.5, 
		self.screenres*472.5, self.screenres*607.5, self.screenres*742.5, self.screenres*877.5, self.screenres*1012.5, self.screenres*1147.5,
		self.screenres*1282.5, self.screenres*1417.5, self.screenres*1552.5]	

		# List of the square positions on y-axis
			
		self.ySquarePositionList = [self.screenres*1012.5+44, self.screenres*877.5+44, self.screenres*742.5+44, self.screenres*607.5+44, 
		self.screenres*472.5+44, self.screenres*337.5+44, self.screenres*202.5+44, self.screenres*67.5+44]

		self.selectedPiece = None # currently selected Piece for moving
		

		
	# Methods used to report the moveability and position of the pieces.
	
	def getMoveableList(self):
		return self.moveableList

	def getVisibleList(self):
		return self.visibleList
		
	def getPositionList(self):
		return self.positionList;
	def setPositionList(self,s):
		self.positionList = s	
	

	# Method used to report whether the trial ends or not.
	def getCompleted(self):
		return self.completed

	def getMoveCount(self):
		return self.moveCount
	

	# Initialize each trial
	# parameter n is the number of disks to be moved in the trial;
	
	def initialize(self, start_pos, visible):
		self.squareDict = dict()
		self.screenres = 0.9185185185185185185185185185185185
		for i in range(8):
			self.squareDict[(self.screenres*67.5, (1080-(self.screenres*67.5+i*self.screenres*135+44)))] = [('A'+str(i))]
			self.squareDict[(self.screenres*202.5, (1080-(self.screenres*67.5+i*self.screenres*135+44)))] = [('B'+str(i))]
			self.squareDict[(self.screenres*337.5, (1080-(self.screenres*67.5+i*self.screenres*135+44)))] = [('C'+str(i))]
			self.squareDict[(self.screenres*472.5, (1080-(self.screenres*67.5+i*self.screenres*135+44)))] = [('D'+str(i))]
			self.squareDict[(self.screenres*607.5, (1080-(self.screenres*67.5+i*self.screenres*135+44)))] = [('E'+str(i))]
			self.squareDict[(self.screenres*742.5, (1080-(self.screenres*67.5+i*self.screenres*135+44)))] = [('F'+str(i))]
			self.squareDict[(self.screenres*877.5, (1080-(self.screenres*67.5+i*self.screenres*135+44)))] = [('G'+str(i))]
			self.squareDict[(self.screenres*1012.5,(1080-(self.screenres*67.5+i*self.screenres*135+44)))] = [('H'+str(i))]
			self.squareDict[(self.screenres*1147.5,(1080-(self.screenres*67.5+i*self.screenres*135+44)))] = ['Start_1']
			self.squareDict[(self.screenres*1282.5,(1080-(self.screenres*67.5+i*self.screenres*135+44)))] = ['Start_2']
			self.squareDict[(self.screenres*1417.5,(1080-(self.screenres*67.5+i*self.screenres*135+44)))] = ['Start_3']
			self.squareDict[(self.screenres*1552.5,(1080-(self.screenres*67.5+i*self.screenres*135+44)))] = ['Start_4']
		self.positionList = list(start_pos)
		self.visibleList = list(visible)
		for i in range(len(self.positionList)):
			if self.visibleList[i]:
				self.squareDict[tuple(self.positionList[i])] = ['test', i]
		self.moveCount = 0
		self.moveableList = list()
		self.source = list()
		self.destine = list()
		self.completed = 0
		self.numberPieces = 32;		
		self.selectedPiece = None
		for i in range(self.numberPieces):
			self.moveableList.append(False)
		
	# Method used to determine which item to pickup
	def processPickup(self, mouseX, mouseY):
		absolute_difference_function = lambda list_value : abs(list_value - mouseX)
		x_pos = min(self.xSquarePositionList, key=absolute_difference_function)

		absolute_difference_function = lambda list_value : abs(list_value - mouseY)
		y_pos = min(self.ySquarePositionList, key=absolute_difference_function)
		self.source = (x_pos, y_pos)
		
		if len(self.squareDict[self.source]) > 1:
			self.selectedPiece = self.squareDict[self.source].pop(1)
			self.moveableList[self.selectedPiece] = True
		else:
			self.selectedPiece = None

	# Method used to determine where to dropoff the selected piece
	def processDropoff(self, mouseX, mouseY):
		absolute_difference_function = lambda list_value : abs(list_value - mouseX)
		x_pos = min(self.xSquarePositionList, key=absolute_difference_function)

		absolute_difference_function = lambda list_value : abs(list_value - mouseY)
		y_pos = min(self.ySquarePositionList, key=absolute_difference_function)
		self.destine = (x_pos, y_pos)

		
		# If there is a piece in the source and the square is empty, move the piece
		if self.selectedPiece is not None:
			if len(self.squareDict[self.destine]) < 2:
				self.positionList[self.selectedPiece] = self.destine
				self.moveableList[self.selectedPiece] = False
				self.squareDict[self.destine].append(self.selectedPiece)
				self.moveCount += 1
			else:
				if x_pos > 992:
					found = False
					for i in self.squareDict:
						if i[0] > 992 and len(self.squareDict[i]) < 2:
							if not found:
								self.positionList[self.selectedPiece] = i
								self.moveableList[self.selectedPiece] = False
								self.squareDict[i].append(self.selectedPiece)
								self.moveCount += 1
								found = True
				else:
					self.squareDict[self.source].append(self.selectedPiece)
					self.moveableList[self.selectedPiece] = False


	# Method used to determine where to dropoff the selected piece and being able to strike a piece in the selected dropoff location
	def processDropoff2(self, mouseX, mouseY):
		absolute_difference_function = lambda list_value : abs(list_value - mouseX)
		x_pos = min(self.xSquarePositionList[4::], key=absolute_difference_function)

		absolute_difference_function = lambda list_value : abs(list_value - mouseY)
		y_pos = min(self.ySquarePositionList, key=absolute_difference_function)
		self.destine = (x_pos, y_pos)
		
		# If there is a piece in the source and the square is empty, move the piece
		if self.selectedPiece is not None:
			if self.visibleList[self.selectedPiece] and self.source != self.destine:
				if len(self.squareDict[self.destine]) < 2:
					self.positionList[self.selectedPiece] = self.destine
					self.moveableList[self.selectedPiece] = False
					self.squareDict[self.destine].append(self.selectedPiece)
					self.moveCount += 1
				else:
					self.visibleList[self.squareDict[self.destine][1]] = False
					self.squareDict[self.destine][1] = self.selectedPiece
					self.positionList[self.selectedPiece] = self.destine
					self.moveableList[self.selectedPiece] = False
					self.moveCount += 1
			else:
				self.moveableList[self.selectedPiece] = False
				self.squareDict[self.source].append(self.selectedPiece)
